@@ -11,6 +11,7 @@ from hiutils.core import (
     make_interactions, cv,
     has_unit_leverage
 )
+from importlib.metadata import version as pkg_version, PackageNotFoundError
 
 
 class HIREG(BaseEstimator):
@@ -52,6 +53,16 @@ class HIREG(BaseEstimator):
     vars_names : list of str, optional
         Names of the main effects variables/features.
 
+    zero_cutoff : float, optional
+        All pvalues below this threshold are considered zero. Defaults to 1e-20.
+
+    interactions_threshold : float, optional
+        The p-value threshold used to select interactions. If None, it will be determined using the largest log-gap rule.
+
+    save_plots : bool, optional
+        If true, save the plots as PNG files. Defaults to False.
+        
+
 
     Attributes
     ----------
@@ -121,6 +132,15 @@ class HIREG(BaseEstimator):
 
     vars_names\_ : list of str, optional
         Names of the main effects variables/features.
+
+    zero_cutoff\_ : float
+        All pvalues below this threshold are considered zero. Defaults to 1e-20.
+
+    interactions_threshold\_ : float
+        The p-value threshold used to select interactions. If None, it will be determined using the largest log-gap rule.
+
+    save_plots\_ : bool
+        If true, save the plots as PNG files. Defaults to False.
     """
 
     def __init__(
@@ -136,6 +156,9 @@ class HIREG(BaseEstimator):
         interaction_candidates=None,
         interaction_pairs=None,
         vars_names=None,
+        zero_cutoff=1e-20,
+        interactions_threshold=None,
+        save_plots=False,
     ):
         
         """
@@ -176,6 +199,14 @@ class HIREG(BaseEstimator):
         vars_names : list of str, optional
             Names of the main effects variables/features.
 
+        zero_cutoff : float, optional
+            All pvalues below this threshold are considered zero. Defaults to 1e-20.
+
+        interactions_threshold : float, optional
+            The p-value threshold used to select interactions. If None, it will be determined using the largest log-gap rule.
+
+        save_plots : bool, optional
+            If true, save the plots as PNG files. Defaults to False.
 
         Attributes
         ----------
@@ -245,6 +276,15 @@ class HIREG(BaseEstimator):
 
         vars_names\_ : list of str, optional
             Names of the main effects variables/features.
+
+        zero_cutoff\_ : float
+            All pvalues below this threshold are considered zero. Defaults to 1e-20.
+
+        interactions_threshold\_ : float
+            The p-value threshold used to select interactions. If None, it will be determined using the largest log-gap rule.
+        
+        save_plots\_ : bool
+            If true, save the plots as PNG files. Defaults to False.
         """
         if hierarchy not in (None, "weak", "strong"):
             raise ValueError("hierarchy must be one of None, 'weak', or 'strong'.")
@@ -292,6 +332,17 @@ class HIREG(BaseEstimator):
             
         self.vars_names_ = vars_names
         self.instable_pairs_ = None
+        self.zero_cutoff_ = zero_cutoff
+        self.interactions_threshold_ = interactions_threshold
+        self.save_plots_ = save_plots
+        
+    @property
+    def version(self):
+        try:
+            return pkg_version("hireg-stats")
+        except PackageNotFoundError:
+            from hireg_stats import __version__ as local_version
+            return local_version
         
     def regress_main_effects(self, X, y, lmda_path=None, tolerance=1e-10):
         """
@@ -318,6 +369,7 @@ class HIREG(BaseEstimator):
 
         self.main_effects_names_ = self.vars_names_ if self.vars_names_ is not None else [f"X{j}" for j in range(self.n_features_)]
         self.main_effects_regressor_.set_vars_names(self.main_effects_names_)
+        save_plots = "main_effects_cv_curve_unilasso.png" if self.save_plots_ else None
         cv_results = cv(
             base=self.main_effects_regressor_,
             X=X,
@@ -325,6 +377,7 @@ class HIREG(BaseEstimator):
             n_folds=self.n_folds_main_effects_,
             lmda_path=lmda_path,
             plot_cv_curve=self.plot_cv_curve_,
+            save_plots=save_plots,
             cv1se=self.cv1se_,
         )  # self.main_effects_regressor_ is fitted in-place after CV
         self.lmda_path_main_effects_ = cv_results["lmda_path"]
@@ -462,30 +515,48 @@ class HIREG(BaseEstimator):
         else:
             raise ValueError("incorrect value for hierarchy")
         
+        self.pvals_ = {(self.main_effects_names_[j],self.main_effects_names_[k]):pval for (j,k),pval in pvals_ea.items()}
+        
         # threshold by the biggest log-gap heuristic
+
         pvals_ea_sorted = sorted(pvals_ea.items(), key=lambda x: x[1], reverse=False)  # ascending order
-        num_zeros_ea = len(pvals_ea_sorted) - len([pval for _, pval in pvals_ea_sorted if pval > 1e-20])
-        tmp = [pval for _, pval in pvals_ea_sorted if pval > 1e-20]
-        if len(tmp) <= 1:
-            selected_pairs_ea = [pair for (pair, _) in pvals_ea_sorted]
-        else:
-            tmp = np.log(np.array(tmp))  # works better than raw p-values
-            i_tmp = np.argmax(tmp[1:] - tmp[:-1]) + num_zeros_ea
-            selected_pairs_ea = [pair for (pair, _) in pvals_ea_sorted[: i_tmp + 1]]
+        num_zeros_ea = len(pvals_ea_sorted) - len([pval for _, pval in pvals_ea_sorted if pval > self.zero_cutoff_])
+
+        if self.interactions_threshold_ is not None:
+            selected_pairs_ea = [pair for (pair,pval) in pvals_ea_sorted if pval<=self.interactions_threshold_]
+            self.threshold_pval_ = self.interactions_threshold_
+        else  :
+            tmp = [pval for _, pval in pvals_ea_sorted if pval > self.zero_cutoff_]
+            if len(tmp) <= 1:
+                selected_pairs_ea = [pair for (pair, _) in pvals_ea_sorted]
+                self.threshold_pval_ = tmp[0] if len(tmp)==1 else 0.0
+            else:
+                tmp = np.log(np.array(tmp))  # works better than raw p-values
+                i_tmp = np.argmax(tmp[1:] - tmp[:-1]) + num_zeros_ea
+                selected_pairs_ea = [pair for (pair, _) in pvals_ea_sorted[: i_tmp + 1]]
+                self.threshold_pval_ = pvals_ea_sorted[i_tmp][1]
 
         if self.plot_cv_curve_:
             mask = np.triu(np.ones_like(pvals, dtype=bool), k=1)
             pv = pvals[mask]
             plt.figure(figsize=(8, 5))
-            plt.hist(pv, bins=int(len(pv) / 20), edgecolor="black")
+            # plt.hist(pv, bins=int(len(pv) / 20), edgecolor="black")
+            plt.hist(-np.log(np.maximum(pv,self.zero_cutoff_)), bins=int(len(pv) / 20), edgecolor="black")
+
 
             sel_pv = [pvals[j, k] for j, k in selected_pairs_ea]
-            plt.scatter(sel_pv, np.zeros_like(sel_pv), color="orange", s=50, label="selected pairs", zorder=10)
+            # plt.scatter(sel_pv, np.zeros_like(sel_pv), color="orange", s=50, label="selected pairs", zorder=10)
+            plt.scatter(-np.log(np.maximum(sel_pv,self.zero_cutoff_)), np.zeros_like(sel_pv), color="orange", s=50, label="selected pairs", zorder=10)
+            title = "Interaction -log(p-values) from triplet models | " 
+            title += "largest log-gap rule" if self.interactions_threshold_ is None else f" threshold={self.interactions_threshold_:.2e}"
+            plt.title(title)
 
-            plt.xlabel("p-value")
+            plt.xlabel("-log(p-value)")
             plt.ylabel("Count")
             plt.legend()
             plt.tight_layout()
+            if self.save_plots_:
+                plt.savefig("interaction_pvalues_histogram.png", dpi=300)
             plt.show()
 
             mask = np.tril(np.ones_like(pvals, dtype=bool), k=0)
@@ -504,17 +575,22 @@ class HIREG(BaseEstimator):
             ax.set_facecolor("white")
             for j, k in selected_pairs_ea:
                 ax.text(k + 0.5, j + 0.5, "★", ha="center", va="center", color="white", fontsize=16, zorder=10)
+            plt.title(title)
             plt.tight_layout()
+            if self.save_plots_:
+                plt.savefig("interaction_pvalues_heatmap.png", dpi=300)
             plt.show()
 
         self.selected_pairs_ = np.array(selected_pairs_ea)
         if self.verbose_:
-            print(f"[Stage 2] Selected {len(self.selected_pairs_)} interaction pairs for cross-validation.")
+            add_on_tmp = "largest log-gap rule" if self.interactions_threshold_ is None else f" threshold={self.interactions_threshold_:.2e}"
+            print(f"[Stage 2] Selected {len(self.selected_pairs_)} interaction pairs for cross-validation ({add_on_tmp}).")
 
         stage2_X, self.interactions_names_ = make_interactions(X, self.selected_pairs_, vars_names=self.main_effects_names_)        # (n_samples, n_pairs)
         stage2_y = y - self.prevalidated_preds_  # (n_samples, )
 
         self.interactions_regressor_.set_vars_names(self.interactions_names_)                                                # list of str
+        save_plots = "interactions_cv_curve_lasso.png" if self.save_plots_ else None
         cv_results = cv(
             base=self.interactions_regressor_,
             X=stage2_X,
@@ -522,6 +598,7 @@ class HIREG(BaseEstimator):
             n_folds=self.n_folds_interactions_,
             lmda_path=lmda_path,
             plot_cv_curve=self.plot_cv_curve_,
+            save_plots=save_plots,
             cv1se=False,
         )
         self.lmda_path_interactions_ = cv_results["lmda_path"]
